@@ -10,6 +10,7 @@
 
 #include <apr_time.h>
 
+#include "gm_scoreboard.h"
 /* Deliberately vary the sleep interval by this percentage: */
 #define SLEEP_RANDOMIZE 5.0
 
@@ -19,6 +20,8 @@ extern hash_t *root;
 
 extern int process_xml(data_source_list_t *, char *);
 
+apr_time_t last_poll_ok, last_poll_failed;
+
 void *
 data_thread ( void *arg )
 {
@@ -29,9 +32,9 @@ data_thread ( void *arg )
    datum_t key;
    char *buf;
    /* This will grow as needed */
-   unsigned int buf_size = 1024, read_index, read_available;
+   unsigned int buf_size = 1024*128, read_index, read_available;
    struct pollfd struct_poll;
-   apr_time_t start, end;
+   apr_time_t start, end, end_poll;
    apr_interval_time_t sleep_time, elapsed;
    double random_factor;
    unsigned int rand_seed;
@@ -130,16 +133,6 @@ data_thread ( void *arg )
                   {
                      if( struct_poll.revents & POLLIN )
                         {
-                           if( (read_index + 1024) > buf_size )
-                              {
-                                 /* We need to malloc more space for the data */
-                                 buf = realloc( buf, buf_size+1024 );
-                                 if(!buf)
-                                    {
-                                       err_quit("data_thread() unable to malloc enough room for [%s] XML", d->name);
-                                    }
-                                 buf_size+=1024;
-                              }
                            if(ioctl(sock->sockfd, FIONREAD, &read_available) == -1)
                            {
                                  err_msg("data_thread() unable to ioctl(FIONREAD) socket for [%s] data source", d->name);
@@ -147,7 +140,16 @@ data_thread ( void *arg )
                                  d->dead = 1;
                                  goto take_a_break;
                            }
-                           read_available = read_available > 1023 ? 1023 : read_available;
+                           if( (read_index + read_available) > buf_size )
+                              {
+                                 /* We need to malloc more space for the data */
+                                 buf = realloc( buf, (buf_size+read_available)*2 );
+                                 if(!buf)
+                                    {
+                                       err_quit("data_thread() unable to malloc enough room for [%s] XML", d->name);
+                                    }
+                                 buf_size=(buf_size+read_available)*2;
+                              }
                            bytes_read = read(sock->sockfd, buf+read_index, read_available);
                            if (bytes_read < 0)
                               {
@@ -205,7 +207,7 @@ data_thread ( void *arg )
 		 err_msg("GZIP compressed data for [%s] data source, %d bytes", d->name, read_index);
 	       }
 
-	     uncompressed = malloc(buf_size);
+	     uncompressed = malloc(buf_size * 2);
 	     if( !uncompressed )
 	       {
 		 err_quit("data_thread() unable to malloc enough room for [%s] GZIP", d->name);
@@ -232,9 +234,9 @@ data_thread ( void *arg )
 	     while (1)
 	       {
 		 /* Create more buffer space if needed */
-		 if ( (write_index + 2048) > buf_size)
+		 if ( (write_index * 2) > buf_size)
 		   {
-		     buf_size += 2048;
+		     buf_size *= 2;
 		     uncompressed = realloc(uncompressed, buf_size);
 		     if(!uncompressed)
 		       {
@@ -284,6 +286,7 @@ data_thread ( void *arg )
 
          buf[read_index] = '\0';
 
+         end_poll = apr_time_now();
          /* Parse the buffer */
          rval = process_xml(d, buf);
          if(rval)
@@ -298,6 +301,19 @@ data_thread ( void *arg )
          d->dead = 0;
 
        take_a_break:
+        if(d->dead)
+	    {
+               end_poll = apr_time_now();
+               ganglia_scoreboard_inc(DS_POLL_FAILED_REQS);
+               ganglia_scoreboard_incby(DS_POLL_FAILED_DURATION, end_poll - start);
+               last_poll_failed = end_poll;  //Updating global variable
+            }
+        else
+	    {
+               ganglia_scoreboard_inc(DS_POLL_OK_REQS);
+               ganglia_scoreboard_incby(DS_POLL_OK_DURATION, end_poll - start);
+               last_poll_ok = end_poll;  //Updating global variable
+            }
          g_tcp_socket_delete(sock);
 
          end = apr_time_now();
@@ -307,6 +323,8 @@ data_thread ( void *arg )
          sleep_time = apr_time_from_sec(d->step) * random_factor - elapsed;
          if(sleep_time > 0)
            apr_sleep(sleep_time); 
+         else
+           ganglia_scoreboard_inc(DS_POLL_MISS);
       }
    return NULL;
 }
