@@ -14,12 +14,14 @@
 
 #include "daemon_init.h"
 #include "update_pidfile.h"
+#include "gm_scoreboard.h"
 
 #include "rrd_helpers.h"
 #include "export_helpers.h"
 
 #define METADATA_SLEEP_RANDOMIZE 5.0
 #define METADATA_MINIMUM_SLEEP 1
+#define HOSTNAMESZ 64
 
 /* Holds our data sources. */
 hash_t *sources;
@@ -41,6 +43,16 @@ extern struct type_tag* in_type_list (char *, unsigned int);
 
 extern g_udp_socket *carbon_udp_socket;
 
+#ifdef WITH_RIEMANN
+extern g_udp_socket *riemann_udp_socket;
+extern g_tcp_socket *riemann_tcp_socket;
+
+extern void *circuit_breaker_thread(void *arg);
+extern int riemann_circuit_breaker;
+extern apr_time_t riemann_reset_timeout;
+extern int riemann_failures;
+#endif /* WITH_RIEMANN */
+
 struct gengetopt_args_info args_info;
 
 extern gmetad_config_t gmetad_config;
@@ -49,6 +61,17 @@ static int debug_level;
 /* In cleanup.c */
 extern void *cleanup_thread(void *arg);
 
+/* The global context */
+apr_pool_t *global_context = NULL;
+
+/* When this gmetad was started */
+apr_time_t started;
+
+/* Last gmetad run */
+apr_time_t last_metadata;
+
+char hostname[HOSTNAMESZ];
+
 static int
 print_sources ( datum_t *key, datum_t *val, void *arg )
 {
@@ -199,26 +222,79 @@ do_root_summary( datum_t *key, datum_t *val, void *arg )
    if (source->ds->dead)
       return 0;
 
-   /* We skip metrics not to be summarized. */
-   if (llist_search(&(gmetad_config.unsummarized_metrics), (void *)key->data, llist_strncmp, &le) == 0)
-      return 0;
-
    /* Need to be sure the source has a complete sum for its metrics. */
    pthread_mutex_lock(source->sum_finished);
 
-   /* We know that all these metrics are numeric. */
-   rc = hash_foreach(source->metric_summary, sum_metrics, arg);
+   if (gmetad_config.summarized_metrics != NULL) {
+      for (le = gmetad_config.summarized_metrics; le != NULL; le = le->next) {
+         datum_t skey, *r;
+         
+         skey.data = le->val;
+         skey.size = strlen(le->val) + 1;
+
+         r = hash_lookup(&skey, source->metric_summary);
+
+         if (r != NULL) {
+            sum_metrics(&skey, r, NULL);
+            datum_free(r);
+         }
+      }
+
+      rc = 0;
+   } else {
+      /* We skip metrics not to be summarized. */
+      if (llist_search(&(gmetad_config.unsummarized_metrics), (void *)key->data, llist_strncmp, &le) == 0) {
+         rc = 0;
+         goto out;
+      }
+
+      /* We know that all these metrics are numeric. */
+      rc = hash_foreach(source->metric_summary, sum_metrics, arg);
+   }
 
    /* Update the top level root source */
    root.hosts_up += source->hosts_up;
    root.hosts_down += source->hosts_down;
 
+out:
    /* summary completed for source */
    pthread_mutex_unlock(source->sum_finished);
 
    return rc;
 }
 
+void initialize_scoreboard()
+{
+    ganglia_scoreboard_init(global_context);
+
+    ganglia_scoreboard_add(DS_POLL_OK_REQS, GSB_COUNTER);
+    ganglia_scoreboard_add(DS_POLL_OK_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(DS_POLL_FAILED_REQS, GSB_COUNTER);
+    ganglia_scoreboard_add(DS_POLL_FAILED_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(DS_POLL_MISS, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_RECVD_ALL, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SENT_ALL, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SENT_RRDTOOL, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SENT_RRDCACHED, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SENT_GRAPHITE, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SENT_MEMCACHED, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SENT_RIEMANN, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_ALL_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_RRDTOOLS_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_RRDCACHED_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_GRAPHITE_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_MEMCACHED_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_RIEMANN_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SUMRZ_ROOT, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SUMRZ_CLUSTER, GSB_COUNTER);
+    ganglia_scoreboard_add(METS_SUMRZ_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(TCP_REQS_ALL, GSB_COUNTER);
+    ganglia_scoreboard_add(TCP_REQS_ALL_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(TCP_REQS_XML, GSB_COUNTER);
+    ganglia_scoreboard_add(TCP_REQS_XML_DURATION, GSB_COUNTER);
+    ganglia_scoreboard_add(TCP_REQS_INTXML, GSB_COUNTER);
+    ganglia_scoreboard_add(TCP_REQS_INTXML_DURATION, GSB_COUNTER);
+}
 
 static int
 write_root_summary(datum_t *key, datum_t *val, void *arg)
@@ -230,6 +306,7 @@ write_root_summary(datum_t *key, datum_t *val, void *arg)
    int rc;
    struct type_tag *tt;
    llist_entry *le;
+   char *p;
 
    name = (char*) key->data;
    metric = (Metric_t*) val->data;
@@ -245,6 +322,11 @@ write_root_summary(datum_t *key, datum_t *val, void *arg)
    if (llist_search(&(gmetad_config.unsummarized_metrics), (void *)key->data, llist_strncmp, &le) == 0)
        return 0;
 
+   /* Don't write a summary for metris that appears to be sFlow VM metrics */
+   if (gmetad_config.unsummarized_sflow_vm_metrics && (p = strchr(name, '.')) != NULL && *(p+1) == 'v')
+     return 0;
+
+   ganglia_scoreboard_inc(METS_SUMRZ_ROOT);
    /* We log all our sums in double which does not suffer from
       wraparound errors: for example memory KB exceeding 4TB. -twitham */
    sprintf(sum, "%.5f", metric->val.d);
@@ -267,11 +349,10 @@ write_root_summary(datum_t *key, datum_t *val, void *arg)
    return 0;
 }
 
-#define HOSTNAMESZ 64
-
 int
 main ( int argc, char *argv[] )
 {
+   int rc;
    struct stat struct_stat;
    pthread_t pid;
    pthread_attr_t attr;
@@ -280,16 +361,28 @@ main ( int argc, char *argv[] )
    mode_t rrd_umask;
    char * gmetad_username;
    struct passwd *pw;
-   char hostname[HOSTNAMESZ];
    gmetad_config_t *c = &gmetad_config;
    apr_interval_time_t sleep_time;
-   apr_time_t last_metadata;
+   apr_time_t summary_started, now;
    double random_sleep_factor;
    unsigned int rand_seed;
 
+   rc = apr_initialize();
+   if (rc != APR_SUCCESS) {
+      return -1;
+   }
+
+   /* create a memory pool. */
+   apr_pool_create(&global_context, NULL);
+
    /* Ignore SIGPIPE */
    signal( SIGPIPE, SIG_IGN );
 
+   initialize_scoreboard();
+
+   /* Mark the time this gmetad started */
+   started = apr_time_now();
+
    if (cmdline_parser(argc, argv, &args_info) != 0)
       err_quit("command-line parser error");
 
@@ -437,6 +530,31 @@ main ( int argc, char *argv[] )
          debug_msg("carbon forwarding ready to send via %s to %s:%d", c->carbon_protocol, c->carbon_server, c->carbon_port);
       }
 
+#ifdef WITH_RIEMANN
+    if (c->riemann_server !=NULL)
+      {
+         if (!strcmp(c->riemann_protocol, "udp"))
+            {
+               riemann_udp_socket = init_riemann_udp_socket (c->riemann_server, c->riemann_port);
+
+               if (riemann_udp_socket == NULL)
+                  err_quit("[riemann] %s socket failed for %s:%d", c->riemann_protocol, c->riemann_server, c->riemann_port);
+            } else if (!strcmp(c->riemann_protocol, "tcp")) {
+                riemann_tcp_socket = init_riemann_tcp_socket (c->riemann_server, c->riemann_port);
+                if (riemann_tcp_socket == NULL) {
+                   riemann_circuit_breaker = RIEMANN_CB_OPEN;
+                   riemann_reset_timeout = apr_time_now () + RIEMANN_RETRY_TIMEOUT * APR_USEC_PER_SEC;
+                } else {
+                   riemann_circuit_breaker = RIEMANN_CB_CLOSED;
+                   riemann_failures = 0;
+                }
+            } else {
+                err_quit("ERROR: Riemann protocol must be 'udp' or 'tcp'");
+            }
+         debug_msg("[riemann] Ganglia configured to forward metrics via %s to %s:%d", c->riemann_protocol, c->riemann_server, c->riemann_port);
+      }
+#endif /* WITH_RIEMANN */
+
    /* initialize summary mutex */
    root.sum_finished = (pthread_mutex_t *) 
                           malloc(sizeof(pthread_mutex_t));
@@ -459,8 +577,16 @@ main ( int argc, char *argv[] )
    pthread_create(&pid, &attr, cleanup_thread, (void *) NULL);
    debug_msg("cleanup thread has been started");
 
+#ifdef WITH_RIEMANN
+   if (!strcmp(c->riemann_protocol, "tcp")) {
+      /* A thread to re-poll riemann TCP port if circuit breaker tripped */
+      pthread_create(&pid, &attr, circuit_breaker_thread, (void *) NULL);
+      debug_msg ("[riemann] circuit breaker thread has been started");
+   }
+#endif /* WITH_RIEMANN */
+
     /* Meta data */
-   last_metadata = 0;
+   last_metadata = apr_time_now();  //Updating global variable
    for(;;)
       {
          /* Do at a random interval, between 
@@ -475,6 +601,7 @@ main ( int argc, char *argv[] )
          /* Need to be sure root is locked while doing summary */
          pthread_mutex_lock(root.sum_finished);
 
+         summary_started = apr_time_now();
          /* Flush the old values */
          hash_foreach(root.metric_summary, zero_out_summary, NULL);
          root.hosts_up = 0;
@@ -490,8 +617,13 @@ main ( int argc, char *argv[] )
          hash_foreach(root.metric_summary, write_root_summary, NULL);
 
          /* Remember our last run */
-         last_metadata = apr_time_now();
+         now = apr_time_now();
+         last_metadata = now;  //Updating global variable
+         ganglia_scoreboard_incby(METS_SUMRZ_DURATION, now - summary_started);
       }
 
+   apr_pool_destroy(global_context);
+
+   apr_terminate();
    return 0;
 }
