@@ -5,6 +5,7 @@
 #include <expat.h>
 #include <ganglia.h>
 #include "gmetad.h"
+#include "gm_scoreboard.h"
 #include "rrd_helpers.h"
 #include "export_helpers.h"
 
@@ -18,6 +19,12 @@ extern struct type_tag* in_type_list (const char *, unsigned int);
 extern Source_t root;
 extern gmetad_config_t gmetad_config;
 
+#ifdef WITH_RIEMANN
+#include "riemann.pb-c.h"
+__thread Msg *riemann_msg = NULL;
+__thread int riemann_num_events;
+#endif /* WITH_RIEMANN */
+
 /* The report method functions (in server.c). */
 extern int metric_report_start(Generic_t *self, datum_t *key, 
                 client_t *client, void *arg);
@@ -524,6 +531,37 @@ startElement_HOST(void *data, const char *el, const char **attr)
       }
    host->stringslen = edge;
 
+#ifdef WITH_RIEMANN
+
+   /* Forward heartbeat metric to Riemann */
+   if (gmetad_config.riemann_server) {
+
+      if (!strcmp(gmetad_config.riemann_protocol, "tcp")) {
+         riemann_msg = malloc (sizeof (Msg));
+         msg__init (riemann_msg);
+      }
+
+      char value[12];
+      sprintf(value, "%d", reported);
+
+      Event *event = create_riemann_event (gmetad_config.gridname, xmldata->sourcename, xmldata->hostname,
+                                    getfield(host->strings, host->ip), "heartbeat", value, "int",
+                                    "seconds", NULL, xmldata->source.localtime, getfield(host->strings, host->tags),
+                                    getfield(host->strings, host->location), tmax * 4);
+
+      if (event) {
+         if (!strcmp(gmetad_config.riemann_protocol, "udp")) {
+             send_event_to_riemann (event);
+         } else {
+             riemann_num_events++;
+             riemann_msg->events = malloc (sizeof (Event) * riemann_num_events);
+             riemann_msg->n_events = riemann_num_events;
+             riemann_msg->events[riemann_num_events - 1] = event;
+         }
+      }
+    }
+#endif /* WITH_RIEMANN */
+
    /* Trim structure to the correct length. */
    hashval.size = sizeof(*host) - GMETAD_FRAMESIZE + host->stringslen;
    hashval.data = host;
@@ -586,6 +624,7 @@ startElement_METRIC(void *data, const char *el, const char **attr)
    const char *name = NULL;
    const char *metricval = NULL;
    const char *type = NULL;
+   const char *units = NULL;
    int do_summary;
    int i, edge, carbon_ret;
    hash_t *summary;
@@ -612,6 +651,9 @@ startElement_METRIC(void *data, const char *el, const char **attr)
                case TYPE_TAG:
                   type = attr[i+1];
                   break;
+               case UNITS_TAG:
+                  units = attr[i+1];
+                  break;
                case SLOPE_TAG:
                   slope = cstr_to_slope(attr[i+1]);
                default:
@@ -639,6 +681,50 @@ startElement_METRIC(void *data, const char *el, const char **attr)
 	 if (metric->dmax && metric->tn > metric->dmax)
             return 0;
 
+#ifdef WITH_RIEMANN
+         /* Forward all metrics, including strings, to Riemann */
+        if (gmetad_config.riemann_server) {
+
+            Host_t *host = (Host_t*) host;
+            host = &(xmldata->host);
+            Event *event;
+
+            if (tt->type == INT || tt->type == UINT) {
+               event = create_riemann_event (gmetad_config.gridname, xmldata->sourcename, xmldata->hostname,
+                                              getfield(host->strings, host->ip), name, metricval, "int",
+                                              units, NULL, xmldata->source.localtime, getfield(host->strings, host->tags),
+                                              getfield(host->strings, host->location), metric->tmax);
+            } else if (tt->type == FLOAT) {
+               event = create_riemann_event (gmetad_config.gridname, xmldata->sourcename, xmldata->hostname,
+                                              getfield(host->strings, host->ip), name, metricval, "float",
+                                              units, NULL, xmldata->source.localtime, getfield(host->strings, host->tags),
+                                              getfield(host->strings, host->location), metric->tmax);
+            } else {
+               event = create_riemann_event (gmetad_config.gridname, xmldata->sourcename, xmldata->hostname,
+                                              getfield(host->strings, host->ip), name, metricval, "string",
+                                              units, NULL, xmldata->source.localtime, getfield(host->strings, host->tags),
+                                              getfield(host->strings, host->location), metric->tmax);
+            }
+            if (event) {
+                if (!strcmp(gmetad_config.riemann_protocol, "udp")) {
+                    send_event_to_riemann (event);
+                } else {
+                    riemann_num_events++;
+                    riemann_msg->events = realloc (riemann_msg->events, sizeof (Event) * riemann_num_events);
+                    riemann_msg->n_events = riemann_num_events;
+                    riemann_msg->events[riemann_num_events - 1] = event;
+                }
+            }
+        }
+#endif /* WITH_RIEMANN */
+
+#ifdef WITH_MEMCACHED
+	/* Put all metrics into memcache including strings */
+	if (gmetad_config.memcached_parameters && !xmldata->ds->dead ) {
+	    int mc_ret=write_data_to_memcached(xmldata->sourcename, xmldata->hostname, name, metricval, xmldata->source.localtime, metric->dmax);
+	}
+#endif /* WITH_MEMCACHED */
+
          if (do_summary && !xmldata->ds->dead && !xmldata->rval)
             {
                   debug_msg("Updating host %s, metric %s", 
@@ -649,13 +735,8 @@ startElement_METRIC(void *data, const char *el, const char **attr)
                         xmldata->ds->step, xmldata->source.localtime, slope);
 		  if (gmetad_config.carbon_server) // if the user has specified a carbon server, send the metric to carbon as well
                      carbon_ret=write_data_to_carbon(xmldata->sourcename, xmldata->hostname, name, metricval,xmldata->source.localtime);
-#ifdef WITH_MEMCACHED
-		  if (gmetad_config.memcached_parameters) {
-                     int mc_ret=write_data_to_memcached(xmldata->sourcename, xmldata->hostname, name, metricval, xmldata->source.localtime, metric->dmax);
-		  }
-
-#endif /* WITH_MEMCACHED */
             }
+
          metric->id = METRIC_NODE;
          metric->report_start = metric_report_start;
          metric->report_end = metric_report_end;
@@ -726,6 +807,7 @@ startElement_METRIC(void *data, const char *el, const char **attr)
          rdatum = hash_insert(&hashkey, &hashval, summary);
          if (!rdatum) err_msg("Could not insert %s metric", name);
       }
+   ganglia_scoreboard_inc(METS_RECVD_ALL);
    return 0;
 }
 
@@ -1072,6 +1154,7 @@ finish_processing_source(datum_t *key, datum_t *val, void *arg)
    Metric_t *metric;
    struct type_tag *tt;
    llist_entry *le;
+   char *p;
 
    name = (char*) key->data;
    metric = (Metric_t*) val->data;
@@ -1088,6 +1171,10 @@ finish_processing_source(datum_t *key, datum_t *val, void *arg)
    if (llist_search(&(gmetad_config.unsummarized_metrics), (void *)name, llist_strncmp, &le) == 0)
       return 0;
 
+   /* Don't save to RRD if this metrics appears to be an sFlow VM metrics */
+   if (gmetad_config.unsummarized_sflow_vm_metrics && (p = strchr(name, '.')) != NULL && *(p+1) == 'v')
+       return 0;
+
    switch (tt->type)
       {
          case INT:
@@ -1110,6 +1197,7 @@ finish_processing_source(datum_t *key, datum_t *val, void *arg)
        debug_msg("Writing Summary data for source %s, metric %s",
                  xmldata->sourcename, name);
 
+       ganglia_scoreboard_inc(METS_SUMRZ_CLUSTER);
        xmldata->rval = write_data_to_rrd(xmldata->sourcename, NULL, name,
                                          sum, num, xmldata->ds->step,
                                          xmldata->source.localtime,
@@ -1213,6 +1301,18 @@ endElement_CLUSTER(void *data, const char *el)
    return 0;
 }
 
+#ifdef WITH_RIEMANN
+static int
+endElement_HOST(void *data, const char *el)
+{
+   if (!strcmp(gmetad_config.riemann_protocol, "tcp")) {
+      send_message_to_riemann(riemann_msg);
+      destroy_riemann_msg(riemann_msg);
+      riemann_num_events = 0;
+   }
+   return 0;
+}
+#endif /* WITH_RIEMANN */
 
 static void
 end (void *data, const char *el)
@@ -1233,6 +1333,12 @@ end (void *data, const char *el)
             rc = endElement_CLUSTER(data, el);
             break;
 
+#ifdef WITH_RIEMANN
+         case HOST_TAG:
+            rc = endElement_HOST(data, el);
+            break;
+#endif /* WITH_RIEMANN */
+
          default:
                break;
       }
