@@ -29,6 +29,6 @@ create_mcast_server(apr_pool_t *context, int32_t family, char *mcast_ip, apr_por
 
 apr_socket_t *
 create_tcp_server(apr_pool_t *context, int32_t family, apr_port_t port, char
-                  *bind, char *interface, int blocking);
+                  *bind, char *interface, int blocking, int32_t gzip_output);
 
 #endif
