@@ -2,7 +2,11 @@
 #include "config.h"
 #endif
 
-#include <stdio.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <grp.h>
+#include <pwd.h>
+
 #include "become_a_nobody.h"
 #include <gm_msg.h>
 
@@ -11,7 +15,7 @@ become_a_nobody( const char *username )
 {
    int rval;
    struct passwd *pw;
- 
+
    pw = getpwnam(username);
    if ( pw == NULL )
      {
@@ -19,14 +23,27 @@ become_a_nobody( const char *username )
      }
 
    rval = getuid();
-   if ( rval != pw->pw_uid )  
+   if ( rval != pw->pw_uid )
      {
        if ( rval != 0 )
          {
            err_quit("Must be root to setuid to \"%s\"\n\n", username);
          }
 
-       rval = setuid(pw->pw_uid); 
+       rval = setgid(pw->pw_gid);
+       if ( rval < 0 )
+         {
+           err_quit("exiting. setgid %d error", (int) pw->pw_gid);
+         }
+
+       rval = initgroups(username, pw->pw_gid);
+       if ( rval < 0 )
+         {
+           err_quit("exiting. initgroups '%s', %d error",
+             username, (int) pw->pw_gid);
+         }
+
+       rval = setuid(pw->pw_uid);
        if ( rval < 0 )
          {
            err_quit("exiting. setuid '%s' error", username);
