@@ -2,16 +2,7 @@
 #define HASH__H 1
 
 #include <stddef.h>				  /* For size_t     */
-#include "rdwr.h"
-
-#define READ_LOCK(__hash, __nodeval) \
-pthread_rdwr_rlock_np( &(__hash->node[__nodeval]->rwlock))
-#define READ_UNLOCK(__hash, __nodeval) \
-pthread_rdwr_runlock_np( &(__hash->node[__nodeval]->rwlock))
-#define WRITE_LOCK(__hash, __nodeval) \
-pthread_rdwr_wlock_np( &(__hash->node[__nodeval]->rwlock))
-#define WRITE_UNLOCK(__hash, __nodeval) \
-pthread_rdwr_wunlock_np( &(__hash->node[__nodeval]->rwlock))
+#include <apr_thread_rwlock.h>
 
 #define HASH_FLAG_IGNORE_CASE 1
 
@@ -22,26 +13,22 @@ typedef struct
 } 
 datum_t;
 
-typedef struct bucket
+typedef struct node
 {
    datum_t *key;
    datum_t *val;
-   struct bucket *next;
+   struct node *next;
+   char __pad[8];
 }
-bucket_t;
-
-typedef struct
-{
-   bucket_t *bucket;
-   pthread_rdwr_t rwlock;    
-} 
 node_t;
 
 typedef struct
 {
-  size_t size;
-  node_t **node;
-  int flags;
+   apr_pool_t *lockpool;
+   apr_thread_rwlock_t **lock;
+   size_t size;
+   node_t *node;
+   int flags;
 }
 hash_t;
 
