@@ -1,8 +1,10 @@
 /*
  * This file contains all the metrics gathering code from Windows using cygwin
  * or native Windows calls when possible.
- * 
- * Tested with cygwin 1.5.23-2 in Windows XP Home SP2 (i386)
+ *
+ * Tested in Windows XP Home SP3 (i386)
+ * Tested in Windows Vista Home SP1 (i386)
+ * Tested in Windows Advanced Server 2000 (i386)
  */
 
 #include <stdio.h>
@@ -16,36 +18,26 @@
 #include <iphlpapi.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
+#include <sys/time.h>
 #include <sys/timeb.h>
 #include <mntent.h>
 #include <sys/vfs.h>
 #include <psapi.h>
 
 /* From old ganglia 2.5.x... */
-#include "file.h"
+#include "gm_file.h"
 #include "libmetrics.h"
 /* End old ganglia 2.5.x headers */
 #undef min
 #undef max
 #include "interface.h"
 
-/* Never changes */
-#ifndef BUFFSIZE
-#define BUFFSIZE 8192
-#endif
-char proc_cpuinfo[BUFFSIZE];
+char *proc_cpuinfo = NULL;
 
 char sys_osname[MAX_G_STRING_SIZE];
 char sys_osrelease[MAX_G_STRING_SIZE];
 
-typedef struct {
-  uint32_t last_read;
-  uint32_t thresh;
-  char *name;
-  char buffer[BUFFSIZE];
-} timely_file;
-
-timely_file proc_stat    = { 0, 1, "/proc/stat" };
+timely_file proc_stat = { {0, 0}, 1., "/proc/stat", NULL, BUFFSIZE };
 
 static time_t
 get_netbw(double *in_bytes, double *out_bytes,
@@ -59,25 +51,25 @@ get_netbw(double *in_bytes, double *out_bytes,
   PMIB_IFROW ifrow;
 
   dwSize = sizeof(MIB_IFTABLE);
-  
+
   iftable = (PMIB_IFTABLE) malloc (dwSize);
   while ((ret = GetIfTable(iftable, &dwSize, 1)) == ERROR_INSUFFICIENT_BUFFER) {
      iftable = (PMIB_IFTABLE) realloc (iftable, dwSize);
   }
- 
-  if (ret == NO_ERROR) { 
+
+  if (ret == NO_ERROR) {
 
     ftime ( &timebuffer );
 
     /* scan the interface table */
     for (dwInterface = 0; dwInterface < (iftable -> dwNumEntries); dwInterface++) {
       ifrow = &(iftable -> table[dwInterface]);
-    
+
     /* exclude loopback */
       if ( (ifrow -> dwType != MIB_IF_TYPE_LOOPBACK ) && (ifrow -> dwOperStatus ==MIB_IF_OPER_STATUS_OPERATIONAL ) ) {
         bytes_in += ifrow -> dwInOctets;
         bytes_out += ifrow -> dwOutOctets;
-      
+
         /* does not include multicast traffic (dw{In,Out}NUcastPkts) */
         pkts_in += ifrow -> dwInUcastPkts;
         pkts_out += ifrow -> dwOutUcastPkts;
@@ -96,24 +88,9 @@ get_netbw(double *in_bytes, double *out_bytes,
   return timebuffer.time;
 }
 
-char *update_file(timely_file *tf)
-{
-  int now,rval;
-  now = time(0);
-  if(now - tf->last_read > tf->thresh) {
-    rval = slurpfile(tf->name, tf->buffer, BUFFSIZE);
-    if(rval == SYNAPSE_FAILURE) {
-      err_msg("update_file() got an error from slurpfile() reading %s",
-              tf->name);
-    }
-    else tf->last_read = now;
-  }
-  return tf->buffer;
-}
-
 /*
-** A helper function to determine the number of cpustates in /proc/stat (MKN)
-*/
+ * A helper function to determine the number of cpustates in /proc/stat (MKN)
+ */
 #define NUM_CPUSTATES_24X 4
 #define NUM_CPUSTATES_26X 7
 static unsigned int num_cpustates;
@@ -124,9 +101,11 @@ num_cpustates_func ( void )
    char *p;
    unsigned int i=0;
 
-   proc_stat.last_read=0;
+   proc_stat.last_read.tv_sec = 0;
+   proc_stat.last_read.tv_usec = 0;
    p = update_file(&proc_stat);
-   proc_stat.last_read=0;
+   proc_stat.last_read.tv_sec = 0;
+   proc_stat.last_read.tv_usec = 0;
 
 /*
 ** Skip initial "cpu" token
@@ -146,7 +125,7 @@ num_cpustates_func ( void )
    return i;
 }
 /*
- * This function is called only once by the gmond.  Use to 
+ * This function is called only once by the gmond.  Use to
  * initialize data structures, etc or just return SYNAPSE_SUCCESS;
  */
 g_val_t
@@ -154,14 +133,20 @@ metric_init(void)
 {
    struct utsname u;
    g_val_t rval;
+   char *bp;
 
    num_cpustates = num_cpustates_func();
 
-   rval.int32 = slurpfile("/proc/cpuinfo", proc_cpuinfo, BUFFSIZE);
-   if ( rval.int32 == SYNAPSE_FAILURE ) {
+   bp = proc_cpuinfo;
+   rval.int32 = slurpfile("/proc/cpuinfo", &bp, BUFFSIZE);
+   if (proc_cpuinfo == NULL)
+      proc_cpuinfo = bp;
+
+   if ( rval.int32 == SLURP_FAILURE ) {
          err_msg("metric_init() got an error from slurpfile() /proc/cpuinfo");
+         rval.int32 = SYNAPSE_FAILURE;
          return rval;
-   }  
+   }
 
    if (uname(&u) == -1) {
       strncpy(sys_osname, "unknown", MAX_G_STRING_SIZE);
@@ -172,8 +157,8 @@ metric_init(void)
       strncpy(sys_osrelease, u.release, MAX_G_STRING_SIZE);
       sys_osrelease[MAX_G_STRING_SIZE - 1] = '\0';
    }
-
    rval.int32 = SYNAPSE_SUCCESS;
+
    return rval;
 }
 
@@ -188,12 +173,13 @@ pkts_in_func ( void )
    unsigned long diff;
 
    stamp = get_netbw(NULL, NULL, &in_pkts, NULL);
-   (unsigned long) diff = in_pkts - last_pkts_in;
+   diff = (unsigned long)(in_pkts - last_pkts_in);
    if ( diff && last_stamp ) {
      t = stamp - last_stamp;
      t = diff / t;
-     debug_msg("Returning value: %f\n", t);     
-   } else t = 0;
+     debug_msg("Returning value: %f\n", t);
+   } else
+     t = 0;
 
    val.f = t;
    last_pkts_in = in_pkts;
@@ -213,12 +199,13 @@ pkts_out_func ( void )
    unsigned long diff;
 
    stamp = get_netbw(NULL, NULL, NULL, &out_pkts);
-   (unsigned long) diff = out_pkts - last_pkts_out;
+   diff = (unsigned long)(out_pkts - last_pkts_out);
    if ( diff && last_stamp ) {
      t = stamp - last_stamp;
      t = diff / t;
-     debug_msg("Returning value: %f\n", t);     
-   } else t = 0;
+     debug_msg("Returning value: %f\n", t);
+   } else
+     t = 0;
 
    val.f = t;
    last_pkts_out = out_pkts;
@@ -238,12 +225,13 @@ bytes_out_func ( void )
    unsigned long diff;
 
    stamp = get_netbw(NULL, &out_bytes, NULL, NULL);
-   (unsigned long) diff = out_bytes - last_bytes_out;
+   diff = (unsigned long)(out_bytes - last_bytes_out);
    if ( diff && last_stamp ) {
      t = stamp - last_stamp;
      t = diff / t;
-     debug_msg("Returning value: %f\n", t);     
-   } else t = 0;
+     debug_msg("Returning value: %f\n", t);
+   } else
+     t = 0;
 
    val.f = t;
    last_bytes_out = out_bytes;
@@ -263,12 +251,13 @@ bytes_in_func ( void )
    unsigned long diff;
 
    stamp = get_netbw(&in_bytes, NULL, NULL, NULL);
-   (unsigned long) diff = in_bytes - last_bytes_in;
+   diff = (unsigned long)(in_bytes - last_bytes_in);
    if ( diff && last_stamp ) {
      t = stamp - last_stamp;
      t = diff / t;
-     debug_msg("Returning value: %f\n", t);     
-   } else t = 0;
+     debug_msg("Returning value: %f\n", t);
+   } else
+     t = 0;
 
    val.f = t;
    last_bytes_in = in_bytes;
@@ -289,8 +278,8 @@ cpu_num_func ( void )
       GetSystemInfo(&siSysInfo);
       cpu_num = siSysInfo.dwNumberOfProcessors;
    }
-
    val.uint16 = cpu_num;
+
    return val;
 }
 
@@ -304,9 +293,9 @@ cpu_speed_func ( void )
 #if defined (__i386__) || defined(__ia64__) || defined(__hppa__) || defined(__x86_64__)
    if (! val.uint32 )
       {
-         p = proc_cpuinfo;  
+         p = proc_cpuinfo;
          p = strstr( p, "cpu MHz" );
-         if(p) {
+         if (p) {
            p = strchr( p, ':' );
            p++;
            p = skip_whitespace(p);
@@ -321,7 +310,7 @@ cpu_speed_func ( void )
          int num;
          p = proc_cpuinfo;
          p = strstr( p, "cycle frequency [Hz]" );
-         if(p) {
+         if (p) {
            p = strchr( p, ':' );
            p++;
            p = skip_whitespace(p);
@@ -338,7 +327,7 @@ cpu_speed_func ( void )
       {
          p = proc_cpuinfo;
          p = strstr( p, "clock" );
-         if(p) { 
+         if (p) {
            p = strchr( p, ':' );
            p++;
            p = skip_whitespace(p);
@@ -360,9 +349,9 @@ mem_total_func ( void )
 
    stat.dwLength = sizeof(stat);
 
-   if ( GlobalMemoryStatusEx (&stat)) {
+   if (GlobalMemoryStatusEx(&stat)) {
       size = stat.ullTotalPhys;
-      /* get the value in kB */ 
+      /* get the value in kB */
       val.f =  size / 1024;
    } else {
       val.f = 0;
@@ -381,9 +370,9 @@ swap_total_func ( void )
 
    stat.dwLength = sizeof(stat);
 
-   if ( GlobalMemoryStatusEx (&stat)) {
+   if (GlobalMemoryStatusEx(&stat)) {
       size = stat.ullTotalPageFile;
-      /* get the value in kB */ 
+      /* get the value in kB */
       val.f =  size / 1024;
    } else {
       val.f = 0;
@@ -398,10 +387,10 @@ boottime_func ( void )
    char *p;
    g_val_t val;
 
-   p = update_file(&proc_stat); 
+   p = update_file(&proc_stat);
 
    p = strstr ( p, "btime" );
-   if(p) { 
+   if (p) {
      p = skip_token ( p );
      val.uint32 = atoi ( p );
    } else {
@@ -417,6 +406,7 @@ sys_clock_func ( void )
    g_val_t val;
 
    val.uint32 = time(NULL);
+
    return val;
 }
 
@@ -425,7 +415,7 @@ machine_type_func ( void )
 {
    SYSTEM_INFO siSysInfo;
    g_val_t val;
- 
+
    GetSystemInfo(&siSysInfo);
 
    switch (siSysInfo.wProcessorArchitecture) {
@@ -492,43 +482,44 @@ total_jiffies_func ( void )
    p = skip_whitespace(p);
    user_jiffies = strtod( p, &p );
    p = skip_whitespace(p);
-   nice_jiffies = strtod( p, &p ); 
+   nice_jiffies = strtod( p, &p );
    p = skip_whitespace(p);
-   system_jiffies = strtod( p , &p ); 
+   system_jiffies = strtod( p , &p );
    p = skip_whitespace(p);
    idle_jiffies = strtod( p , &p );
 
    return (user_jiffies + nice_jiffies + system_jiffies + idle_jiffies);
-}   
+}
 
 g_val_t
 cpu_user_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
-   static double last_user_jiffies,  user_jiffies, 
+   static struct timeval stamp = {0, 0};
+   static double last_user_jiffies,  user_jiffies,
                  last_total_jiffies, total_jiffies, diff;
-   
+
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if ((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+       (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
-     
+
      p = skip_token(p);
      user_jiffies  = strtod( p , (char **)NULL );
      total_jiffies = total_jiffies_func();
-     
-     diff = user_jiffies - last_user_jiffies; 
-     
+
+     diff = user_jiffies - last_user_jiffies;
+
      if ( diff )
        val.f = (diff/(total_jiffies - last_total_jiffies))*100;
      else
        val.f = 0.0;
-     
+
      last_user_jiffies  = user_jiffies;
      last_total_jiffies = total_jiffies;
-     
    }
+
    return val;
 }
 
@@ -537,46 +528,48 @@ cpu_nice_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp = {0, 0};
    static double last_nice_jiffies,  nice_jiffies,
                  last_total_jiffies, total_jiffies, diff;
- 
+
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if ((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+       (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
- 
+
      p = skip_token(p);
      p = skip_token(p);
      nice_jiffies  = strtod( p , (char **)NULL );
      total_jiffies = total_jiffies_func();
 
      diff = (nice_jiffies  - last_nice_jiffies);
- 
+
      if ( diff )
        val.f = (diff/(total_jiffies - last_total_jiffies))*100;
      else
        val.f = 0.0;
- 
+
      last_nice_jiffies  = nice_jiffies;
      last_total_jiffies = total_jiffies;
-
    }
+
    return val;
 }
 
-g_val_t 
+g_val_t
 cpu_system_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp = {0, 0};
    static double last_system_jiffies,  system_jiffies,
                  last_total_jiffies, total_jiffies, diff;
- 
+
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if ((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+       (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
-     
+
      p = skip_token(p);
      p = skip_token(p);
      p = skip_token(p);
@@ -592,56 +585,56 @@ cpu_system_func ( void )
      total_jiffies  = total_jiffies_func();
 
      diff = system_jiffies  - last_system_jiffies;
- 
+
      if ( diff )
        val.f = (diff/(total_jiffies - last_total_jiffies))*100;
      else
        val.f = 0.0;
- 
-     last_system_jiffies  = system_jiffies;
-     last_total_jiffies = total_jiffies;   
 
+     last_system_jiffies  = system_jiffies;
+     last_total_jiffies = total_jiffies;
    }
+
    return val;
 }
 
-g_val_t 
+g_val_t
 cpu_idle_func ( void )
 {
    char *p;
    static g_val_t val;
-   static int stamp;
+   static struct timeval stamp = {0, 0};
    static double last_idle_jiffies,  idle_jiffies,
                  last_total_jiffies, total_jiffies, diff;
- 
+
    p = update_file(&proc_stat);
-   if(proc_stat.last_read != stamp) {
+   if ((proc_stat.last_read.tv_sec != stamp.tv_sec) &&
+       (proc_stat.last_read.tv_usec != stamp.tv_usec)) {
      stamp = proc_stat.last_read;
-     
+
      p = skip_token(p);
      p = skip_token(p);
      p = skip_token(p);
      p = skip_token(p);
      idle_jiffies  = strtod( p , (char **)NULL );
      total_jiffies = total_jiffies_func();
-     
+
      diff = idle_jiffies - last_idle_jiffies;
-     
-     if ( diff ) 
+
+     if ( diff )
        val.f = (diff/(total_jiffies - last_total_jiffies))*100;
      else
        val.f = 0.0;
-     
+
      last_idle_jiffies  = idle_jiffies;
      last_total_jiffies = total_jiffies;
-     
    }
-   
+
    return val;
 }
 
 /* FIXME? */
-g_val_t 
+g_val_t
 cpu_aidle_func ( void )
 {
    g_val_t val;
@@ -652,18 +645,18 @@ cpu_aidle_func ( void )
 }
 
 /* FIXME? */
-g_val_t 
+g_val_t
 cpu_wio_func ( void )
 {
    g_val_t val;
- 
+
    val.f = 0.0;
 
    return val;
 }
 
 /* FIXME? */
-g_val_t 
+g_val_t
 cpu_intr_func ( void )
 {
    g_val_t val;
@@ -674,7 +667,7 @@ cpu_intr_func ( void )
 }
 
 /* FIXME? */
-g_val_t 
+g_val_t
 cpu_sintr_func ( void )
 {
    g_val_t val;
@@ -702,7 +695,7 @@ load_five_func ( void )
    g_val_t val;
 
    val.f = 0.0;
- 
+
    return val;
 }
 
@@ -713,10 +706,11 @@ load_fifteen_func ( void )
    g_val_t val;
 
    val.f = 0.0;
- 
+
    return val;
 }
 
+/* FIXME: fixed number of processes */
 #define MAXPROCESSES 1024
 
 /* FIXME */
@@ -724,9 +718,7 @@ g_val_t
 proc_run_func( void )
 {
    DWORD aProcesses[MAXPROCESSES], cbNeeded, cProcesses;
-   unsigned int i, running = 0;
-   HANDLE hProcess;
-   BOOL bResult;
+   unsigned int running = 0;
    g_val_t val;
 
    if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded)) {
@@ -736,10 +728,14 @@ proc_run_func( void )
    }
 #if (_WIN32_WINNT >= 0x0501)
    /* Only for XP or newer */
+   unsigned int i;
+   HANDLE hProcess;
+   BOOL bResult;
+
    for (i = 0; i < cProcesses; i++)
       if (aProcesses[i] != 0) {
-         hProcess = OpenProcess (PROCESS_QUERY_INFORMATION, 
-            FALSE, aProcesses[i]);
+         hProcess = OpenProcess(PROCESS_QUERY_INFORMATION,
+                                FALSE, aProcesses[i]);
          if (hProcess != NULL) {
             if (IsProcessInJob(hProcess, NULL, &bResult)) {
                if (bResult)
@@ -759,7 +755,7 @@ proc_run_func( void )
 g_val_t
 proc_total_func ( void )
 {
-   DWORD aProcesses[MAXPROCESSES], cbNeeded, cProcesses;   
+   DWORD aProcesses[MAXPROCESSES], cbNeeded, cProcesses;
    g_val_t val;
 
    if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded)) {
@@ -767,7 +763,7 @@ proc_total_func ( void )
    } else {
       cProcesses = cbNeeded / sizeof(DWORD);
    }
-   val.uint32 = cProcesses; 
+   val.uint32 = cProcesses;
 
    return val;
 }
@@ -781,7 +777,7 @@ mem_free_func ( void )
 
    stat.dwLength = sizeof(stat);
 
-   if ( GlobalMemoryStatusEx (&stat)) {
+   if (GlobalMemoryStatusEx(&stat)) {
       size = stat.ullAvailPhys;
       /* get the value in kB */
       val.f =  size / 1024;
@@ -835,7 +831,7 @@ swap_free_func ( void )
 
    stat.dwLength = sizeof(stat);
 
-   if ( GlobalMemoryStatusEx (&stat)) {
+   if (GlobalMemoryStatusEx(&stat)) {
       size = stat.ullAvailPageFile;
       /* get the value in kB */
       val.f =  size / 1024;
@@ -846,11 +842,13 @@ swap_free_func ( void )
    return val;
 }
 
-/* --------------------------------------------------------------------------- */
 g_val_t
 mtu_func ( void )
 {
-   /* We want to find the minimum MTU (Max packet size) over all UP interfaces. */
+   /*
+    * We want to find the minimum MTU (Max packet size) over all UP
+    * interfaces.
+    */
    g_val_t val;
 
    val.uint32 = get_min_mtu();
@@ -885,16 +883,16 @@ find_disk_space(double *total, double *avail)
 
    mnttab = setmntent(MOUNTED, "r");
    while ((ent = getmntent(mnttab)) != NULL) {
-      if (islower(ent->mnt_fsname[0])) {
+      if (islower((int)ent->mnt_fsname[0])) {
          drive = ent->mnt_fsname[0] - 'a';
          if (drives[drive].total == 0.0) {
             statfs(ent->mnt_fsname, &fs);
-         
+
             drives[drive].total = (double)fs.f_blocks * fs.f_bsize;
             drives[drive].avail = (double)fs.f_bavail * fs.f_bsize;
-         
-            pct = (drives[drive].avail == 0) ? 100.0 : 
-               ((drives[drive].total - 
+
+            pct = (drives[drive].avail == 0) ? 100.0 :
+               ((drives[drive].total -
                drives[drive].avail)/drives[drive].total) * 100.0;
 
             if (pct > most_full)
@@ -908,7 +906,7 @@ find_disk_space(double *total, double *avail)
    endmntent(mnttab);
 
    return most_full;
-}   
+}
 
 g_val_t
 disk_free_func( void )
@@ -917,7 +915,7 @@ disk_free_func( void )
    double total_size = 0.0;
    g_val_t val;
 
-   find_disk_space(&total_size, &total_free); 
+   find_disk_space(&total_size, &total_free);
 
    val.d = total_free;
    return val;
