@@ -1358,3 +1358,11 @@ part_max_used_func ( void )
    val.f = most_full;
    return val;
 }
+
+g_val_t
+cpu_steal_func ( void )
+{
+   static g_val_t val=0;
+   return val;
+}
+
